# -*- coding: utf-8 -*-

import sys
import logging
import time
import os
from threading import Thread,Lock,currentThread
import urllib
import new

from songanalysiscommon import *
from Observable import Observer
from amarokplus import NotAnalyzedYet,UnAnalyzable

from qt import *
from searchwindow import SearchWindow
from smartdjpreferences import SmartDJPreferences
from taptempo import TapTempo
from amarokplus import DcopAmarokFailed

logger = logging.getLogger("PyQt UI")
amarok = None
config = None
analysisProcess = None
app = None


# this code is now shared between Qt and GTK+ interface, find a way to librarize it so we don't have to modify it everywhere
# get_ui_config_lock = Lock()
# 
# def get_ui_config():
# 	global get_ui_config_lock
# 	get_ui_config_lock.acquire()
# 	try:
# 		c = get_plugin_public_interface().get_config()
# 		def has_key(object,key): return object.has_key(key)
# 		if not has_key(c,"ui"):
# 			c["ui"] = dict()
# 		if not has_key(c["ui"],"SearchWindow"):
# 			c["ui"]["SearchWindow"] = dict()
# 		if not has_key(c["ui"]["SearchWindow"],"height"):
# 			c["ui"]["SearchWindow"]["height"] = 350
# 		if not has_key(c["ui"]["SearchWindow"],"width"):
# 			c["ui"]["SearchWindow"]["width"] = 600
# 		if not has_key(c["ui"]["SearchWindow"],"columns"):
# 			c["ui"]["SearchWindow"]["columns"] = [ ["File name",1,True,False,120],["Folder",2,True,False,300],["Artist",3,True,True,110],["Album",4,True,True,110],["Title",5,True,True,110],["Similarity",6,False,True],["Tempo",7,False,True] ]
# 		if not has_key(c["ui"]["SearchWindow"],"num_results"):
# 			c["ui"]["SearchWindow"]["num_results"] = 10
# 		if not has_key(c["ui"]["SearchWindow"],"tempo_source"):
# 			c["ui"]["SearchWindow"]["tempo_source"] = CHOSEN_SONG_TEMPO
# 		if not has_key(c["ui"]["SearchWindow"],"fixed_tempo"):
# 			c["ui"]["SearchWindow"]["fixed_tempo"] = 120
# 		if not has_key(c["ui"]["SearchWindow"],"weighting"):
# 			c["ui"]["SearchWindow"]["weighting"] = 1.0
# 	
# 		return c["ui"]
# 	finally:
# 		get_ui_config_lock.release()

# async operation!!!
class AsyncOperation(QTimer):
	lock = Lock()
	def __init__(self,parent,time,func_call,*args):
		"""time is in seconds or fractions thereof"""
		self.lock.acquire()
		qApp.lock()
		try:
			QTimer.__init__(self,parent)
			self.func_call = func_call
			self.args = args
			logger.debug("Preparing %s in thread %s",self.func_call.__name__,currentThread())
			QObject.connect(self,SIGNAL("timeout()"),self.on_timeout)
			self.start(int(time*1000),True)
		finally:
			qApp.unlock()
			self.lock.release()
	
	def on_timeout(self):
		logger.debug("Running %s in thread %s",self.func_call.__name__,currentThread())
		args = self.args
		try: self.func_call(*args)
		except: logger.exception("Operation threw an exception")

class SyncOperation(AsyncOperation):
	def __init__(self,parent,func_call,*args):
		AsyncOperation.__init__(self,parent,0,func_call,*args)
# end async operation



def w(object,name): return object.child(name,None,True)

class OTapTempo(TapTempo):
	
	originally_playing_song_state = None
	assigned_track_state = None
	
	def __init__(self,parent):
		TapTempo.__init__(self,parent,fl=Qt.WDestructiveClose)
	
	def assignTrack(self,filename):
		self.filename = filename
		self.song = amarok.collection.getSong(filename)
		
		at = self.song.getShortName()
# 		bt = "<b>%s</b> by <b>%s</b>"%(title,artist)
		self.setCaption(tounicode(at + " - Smart DJ tempo adjustment"))
		w(self,"trackName").setText(tounicode("Adjusting tempo of track: " + at))
		
		self.original_tempo = self.song.tempo
		self.estimated_new_tempo = None
		self.nearest_harmonic_tempo = None
		
		if self.original_tempo is None:
			w(self,"useNearestHarmonicTempo").setEnabled(False)
			w(self,"useEstimatedNewTempo").setChecked(True)
		
		if self.original_tempo is None: tempo = "No original tempo"
		else: tempo = "%.2f BPM"%self.original_tempo
		w(self,"originalTempo").setText(tounicode(tempo))
		
		if self.original_tempo is None:
			w(self,"buttonTap").setText(tounicode("Hit me &at least 32 times to the rhythm!"))
		
		self.originally_playing_song_state = None
		self.assigned_track_state = None
		
		if amarok.player.path() == self.filename: # fixme this needs to update to the tune of amarok state track changes, so we need to observe it
			self.disableCueButton()
			self.enableTapButton()
	
	def disableCueButton(self):
		w(self,"buttonCue").setEnabled(False)
	
	def enableTapButton(self):
		w(self,"buttonTap").setEnabled(True)
		w(self,"buttonTap").setFocus()
	
	def cue(self): # called when cue button is clicked
		self.disableCueButton()
		self.rememberCurrentState()
		self.startPlayingAssignedTrack()
		self.enableTapButton()
		
	def accept(self):
		self.restoreState()
		
		if w(self,"useNearestHarmonicTempo").isChecked():
			assert(self.nearest_harmonic_tempo is not None)
			if self.nearest_harmonic_tempo != self.original_tempo:
				self.song.tempo = self.nearest_harmonic_tempo
		else: self.song.tempo = self.estimated_new_tempo
		
		TapTempo.accept(self)
		
	def reject(self):
		self.restoreState()
		TapTempo.reject(self)
	
	def startPlayingAssignedTrack(self):
		amarok.playlist.playMedia(self.filename)
		amarok.player.seek(amarok.player.trackTotalTime() / 3) # one third of the song
	
	def rememberCurrentState(self):
		
		status = amarok.player.status()
		if status != 0 :
			path = amarok.player.path()
			progress = amarok.player.trackCurrentTime()
			song = amarok.collection.getSong(path)
			score = song.score
			playcount = song.playcount
			self.originally_playing_song_state = (path,progress,song,score,playcount,status)
			
		song = amarok.collection.getSong(self.filename)
		score = song.score
		playcount = song.playcount
		self.assigned_track_state = (song,score,playcount)
	
	def restoreState(self): # we should also restore paused state, or if it was not playing, stop state
		if self.originally_playing_song_state:
			o = self.originally_playing_song_state
			amarok.playlist.playMedia(o[0])
			amarok.player.seek(o[1])
			if o[5] == 1: # paused
				amarok.player.pause()
			o[2].score = o[3]
			o[2].playcount = o[4]
# 		else: # we cannot do this because when I select the track that is playing now to edit the tempo, then originally playing song state does not get assigned, and over here we get to this point and BAM, stop!
# 			amarok.player.stop()
			
		if self.assigned_track_state:
			o = self.assigned_track_state
			o[0].score = o[1]
			o[0].playcount = o[2]
	
	def tap(self): # called when tap button is clicked
		if not hasattr(self,"taps"): self.taps = []
		
		self.taps.append(time.time())
		
		def get_distances(taps):
			distances = []
			last_tap = None
			for tap in taps:
				if last_tap:
					distances.append(tap - last_tap)
				last_tap = tap
			return distances
		
		# if we have an original tempo, we average 8 taps, if we don't we average 32 taps
		mintaps = 8
		if self.original_tempo is None: mintaps = 32
		if len(self.taps) >= 2:
			self.taps = self.taps[-mintaps:]
			distances = get_distances(self.taps)
			logger.debug("Taps distances: %s",distances)
			average_distance = sum(distances)/len(distances)
			self.estimated_new_tempo = (1.0/average_distance)*60
			
			if self.original_tempo is not None:
				o = self.original_tempo
				factors = [1/4.0,2/4.0,3/4.0,1.0,5/4.0,6/4.0,7/4.0,1/3.0,2/3.0,3/3.0,4/3.0,5/3.0,2.0]
				possibilities = [ o * f for f in factors ]
				distances = [ abs(self.estimated_new_tempo - p) for p in possibilities ]
				min_distance = min(distances)
				possibility = possibilities[distances.index(min_distance)]
				self.nearest_harmonic_tempo = possibility
			
			self.updateTempoDisplays()
	
	def updateTempoDisplays(self):
		if self.nearest_harmonic_tempo:
			if self.nearest_harmonic_tempo == self.original_tempo:
				tempo = "unchanged"
			else:
				tempo = "%.2f BPM"%self.nearest_harmonic_tempo
			w(self,"nearestHarmonicTempo").setText(tounicode(tempo))
		if self.estimated_new_tempo:
			tempo = "%.2f BPM"%self.estimated_new_tempo
			w(self,"estimatedNewTempo").setText(tounicode(tempo))
			w(self,"buttonApply").setEnabled(True)


class OSearchWindow(SearchWindow):
	
	def __init__(self,parent):
		SearchWindow.__init__(self,parent,fl=Qt.WDestructiveClose)
		self.__stopsearch = False
		
		resultsView = w(self,"resultsView")
		resultsView.removeColumn(0)
		resultsView.addColumn("Path")
		resultsView.addColumn("File name")
		resultsView.addColumn("Folder")
		resultsView.addColumn("Artist")
		resultsView.addColumn("Album")
		resultsView.addColumn("Title")
		resultsView.addColumn("Similarity")
		resultsView.addColumn("Tempo")
		for a in [0,1,2,3,4,5]: resultsView.setColumnWidthMode(a,QListView.Manual)
		for a in [3,4,5]: resultsView.setColumnWidth(a,155)
		resultsView.setSorting(6,False)
		resultsView.hideColumn(0)
		resultsView.hideColumn(1)
		resultsView.hideColumn(2)
		
		searchwindow = self
		def dragObject(self):
			d = QUriDrag(self)
			urllist = []
			for a in searchwindow.getSelectedTrackPaths():
				path = a
				pathinunicode = tounicode(path)
				url = urllib.basejoin(u"file://",urllib.pathname2url(pathinunicode.encode("utf-8")))
				urllist.append (url)
			print urllist
			d.setUris(urllist)
			return d
		
		resultsView.dragObject = new.instancemethod( dragObject , resultsView, QListView )
		
	def stopSearch(self):
		self.__stopsearch = True
	
	def search(self,filename):
		self.filename = filename
		song = amarok.collection.getSong(filename)
		at = song.getShortName()
		self.setCaption(tounicode(at + " - amaroK Smart DJ search"))
		w(self,"startSong").setText(tounicode(at))
		self.performSearch()
	
	def getSelectedTrackPaths(self):
		# will be returning a list of strs, NOT UNICODEs
		paths = []
		resultsView = w(self,"resultsView")
		item = resultsView.firstChild()
		while item:
			if resultsView.isSelected(item): paths.append(item.path)
			item = item.nextSibling()
		return paths
	
	def searchSelectedTracks(self):
		app.search(self.getSelectedTrackPaths())
	
	def addSelectedTracksToPlaylist(self):
		for a in self.getSelectedTrackPaths():
			amarok.playlist.addMedia(a)
		
	def performSearch(self):
		logger.debug("Searching similar tracks to %s",self.filename)

		start_file = self.filename
		
		def reflectSearching(trueorfalse):
			
			disableduringsearch = [
			"buttonSearch",
			"tempoSpectrumBalance",
			"useSelectedSongTempo",
			"useFixedTempo",
			"numResults",
			]
			enableduringsearch = [
			"buttonStop",
			]
			
			for wi in disableduringsearch:
				w(self,wi).setDisabled(trueorfalse)
			for wi in enableduringsearch:
				w(self,wi).setEnabled(trueorfalse)
				
			if trueorfalse: #if we're searching
				w(self,"fixedTempo").setEnabled(False)
			else:
				w(self,"fixedTempo").setEnabled(w(self,"useFixedTempo").isChecked())
		
		reflectSearching(True)
		resultsView = w(self,"resultsView")
		resultsView.clear()
		
		app.processEvents()
		
		spectrum_weight = w(self,"tempoSpectrumBalance").value() / 10.0
		tempo_weight = 2 - spectrum_weight
		limit = w(self,"numResults").value()
		if w(self,"useFixedTempo").isChecked():
			fixed_tempo = w(self,"fixedTempo").value()
		else: fixed_tempo = None

		logger.debug("Searching for %s max %s results with tempo/spectrum %s/%s",start_file,limit,tempo_weight,spectrum_weight)
		
		
		results = amarok.collection.getSimilarSongs(
			start_file,spectrum_weight,tempo_weight,limit,fixed_tempo=fixed_tempo)
		
		app.processEvents()

		if self.__stopsearch:
			self.__stopsearch = False
			reflectSearching(False)
			return
			
		resultsView.setFocus()
		
		def actual_put(resultsView,r,artist,album,title):
			class MyListViewItem(QListViewItem):
				def key(self,column,ascending):
					if column in [6,7]:
						key = "0" * (10-len(self.text(column))) + str(self.text(column))
						return QString(key)
					else: return self.text(column)
			listitem = MyListViewItem(resultsView,tounicode(r[0]),tounicode(os.path.basename(r[0])),
						tounicode(os.path.dirname(r[0])),tounicode(artist),tounicode(album),
						tounicode(title),"%.2f"%r[1],"%.1f"%r[2])
			listitem.path = r[0] # store the path as a member var
			listitem.setDragEnabled(True)
		
		def do_all(resultsView,results):
			if results: maxcompat = max( [ r[1] for r in results] )
			else: return # no results, we return
			for r in results:
				if self.__stopsearch:
					self.__stopsearch = False
					break
				r[1] = r[1] *100.0 / maxcompat
				song_object =  amarok.collection.getSong(r[0])
				app.processEvents()
				artist = song_object.artist
				app.processEvents()
				album = song_object.album
				app.processEvents()
				title = song_object.title
				app.processEvents()
				actual_put(resultsView,r,artist,album,title)
				app.processEvents()
			
		def _do_all(resultsView,results):
			try: do_all(resultsView,results)
			except DcopAmarokFailed,e:
				logger.debug("amaroK has gone unexpectedly",exc_info=True)
				self.reject() # attempt to close the dialog synchronously
			except: logger.exception("Something went bad")
			
		_do_all(resultsView,results)
		reflectSearching(False)
		

class OSmartDJPreferences(SmartDJPreferences):
	fully_initialized = False
	
	def __init__(self,parent):
		SmartDJPreferences.__init__(self,parent,name="Preferences window",fl=Qt.WDestructiveClose)
		
		c =  config["super_dynamic_mode"]
		
		w(self,"enableAutoDJ").setChecked(c["enabled"])
		w(self,"topChoices").setValue(c["top_choices"])
		w(self,"upcomingSongs").setValue(c["upcoming_songs"])
		
		if c["choice_mechanism"] == SONG_IM_LISTENING:
			w(self,"currentlyListeningSong").setChecked(True)
		elif c["choice_mechanism"] == SONG_IM_LISTENING_WANDER:
			w(self,"currentlyListeningSongWander").setChecked(True)
		elif c["choice_mechanism"] == LAST_SONG_IN_PLAYLIST:
			w(self,"lastSongInPlaylist").setChecked(True)
		elif c["choice_mechanism"] == LAST_SONG_I_LIKED:
			w(self,"lastSongILiked").setChecked(True)
		else: assert(False)
		tempo_weight,spectrum_weight = (c["tempo_weight"],c["spectrum_weight"])
		w(self,"tempoSpectrumBalance").setValue(int(round(spectrum_weight*10,0)))
		
		combobox = w(self,"playlistComboBox")
		def refill(self): # fixme esto debe bloquear las señales del combobox y proceder a hacer el refil, para posteriormente volver a tomar el valor que estaba seleccionado originalmente, SI ES QUE existe, y si no, ninguno.
			self.clear()
			pnames = ["The entire collection"] + amarok.smartPlaylistCollection.keys()
			for pname in pnames: self.insertItem(tounicode(pname))
		combobox.refill = new.instancemethod( refill, combobox, QComboBox )
		
		combobox.refill()
		
		if c["source_playlist"] is None:
			combobox.setCurrentText("The entire collection")
		else:
			combobox.setCurrentText(c["source_playlist"])
		
		QTimer.singleShot(500,self.updateStatus)
# 		timer = QTimer(self)
# 		QObject.connect(timer,SIGNAL("timeout()"),self.updateStatus)
# 		timer.start(5000)
		
		self.fully_initialized = True
	
	def configChangedByUser(self):
		if not self.fully_initialized: return
		logger.debug("Configuration changed by user")
		
		c = config["super_dynamic_mode"]
		
		c["enabled"] = w(self,"enableAutoDJ").isChecked()
		c["top_choices"] = w(self,"topChoices").value()
		c["upcoming_songs"] = w(self,"upcomingSongs").value()
		
		if w(self,"currentlyListeningSong").isChecked():
			c["choice_mechanism"] = SONG_IM_LISTENING
		elif w(self,"currentlyListeningSongWander").isChecked():
			c["choice_mechanism"] = SONG_IM_LISTENING_WANDER
		elif w(self,"lastSongInPlaylist").isChecked():
			c["choice_mechanism"] = LAST_SONG_IN_PLAYLIST
		elif w(self,"lastSongILiked").isChecked():
			c["choice_mechanism"] = LAST_SONG_I_LIKED
		else: assert(False)
		
		c["spectrum_weight"] = float(w(self,"tempoSpectrumBalance").value()) / 10.0
		c["tempo_weight"] = 2.0 - c["spectrum_weight"] 
		
		combobox = w(self,"playlistComboBox")
		
		if combobox.currentItem() == 0:
			c["source_playlist"] = None
		else:
			c["source_playlist"] = str(combobox.currentText())
		
		config.notifyChanged()
		logger.debug("New configuration: %s",c)
	
	
	def updateStatus(self):
		try:
			current_song = analysisProcess.getNextSong()
			app.processEvents()
			pqueue_count = analysisProcess.getPriorityQueueLength()
			app.processEvents()
			total = amarok.collection.totalTracks()
			app.processEvents()
			left = amarok.collection.countUnanalyzedSongs()
			
			if total == 0:
				progress = 1.0
			else:
				progress = ( float(total) - float(left) ) / float(total)
			
			logger.debug("Analysis process report: total: %s, left: %s",total,left)
			
			if current_song:
				w(self,"songCurrentlyAnalyzing").setText(tounicode("Analyzing %s"%current_song))
				string = "%s of %s songs (%.1f%%) pending analysis"%(left,total,100-progress*100)
			else:
				if total == 0:
					w(self,"overallProgressGroupBox").hide()
					w(self,"songCurrentlyAnalyzing").setText(tounicode("No tracks in your collection.  Please rebuild your collection."))
				else:
					w(self,"overallProgressGroupBox").show()
					w(self,"songCurrentlyAnalyzing").setText(tounicode("Idle.  All tracks have been analyzed."))
				string = "%s tracks analyzed"%(total)
			if pqueue_count:
				if pqueue_count != 1: s = "s"
				else: s = ""
				w(self,"priorityQueueLength").setText(tounicode("%s track%s in the analysis priority queue"%(pqueue_count,s)))
			else:
				w(self,"priorityQueueLength").setText(tounicode("No tracks in the analysis priority queue"))
			
			w(self,"analysisProgress").setProgress(int(progress*1000))
			w(self,"analysisProgressText").setText(tounicode(string))
		except DcopAmarokFailed,e:
			logger.debug("amaroK is unexpectedly gone")


# my own qt app

class EventProcessor(Observer):
	
	def __init__(self,parent):
		Observer.__init__(self)
		self.parent = parent
		self.observe(amarok)
		self.observe(analysisProcess)
				
	def notify(self,object,eventName,*args):
		logger.debug( "Processing %s"%eventName)
		if eventName == "customMenuClicked":
			line = args[0]
			logger.debug("Arguments: %s",args)
			s = "Smart DJ Find tracks similar to selection..."
			t = "Smart DJ Set up Auto DJ mode..."
			u = "Smart DJ Analyze selected tracks now"
			v = "Smart DJ Edit tempo of selected tracks..."
			
			def getfilenames(prefix,string):
				fnames = string[len(prefix)+1:]
				fnames = [ urllib.url2pathname(fname)[7:] for fname in fnames.split(" ") ]
				return fnames
			
			if line[:len(s)] == s:
				self.parent.search(getfilenames(s,line))
				
			elif line[:len(u)] == u:
				filenames = getfilenames(u,line)
				for filename in filenames:
					analysisProcess.addToPriorityQueue(filename)
				if len(filenames) == 1:
					thesong = amarok.collection.getSong(filenames[0])
					mm = thesong.getLongName(True)
				else: mm = "<b>%s tracks</b>"%len(filenames)
				amarok.playlist.shortStatusMessage("Added %s to analysis priority queue"%mm)
				
			elif line[:len(v)] == v:
				self.parent.edit_tempo(getfilenames(v,line))
				
			elif line[:len(t)] == t:
				self.parent.show_configuration()
				
		if eventName == "configure":
			self.parent.show_configuration()
	
		if eventName == "trackAnalyzed":
			self.parent.update_config_window()
	
		if eventName == "priorityQueueChanged":
			self.parent.update_config_window()
	
class UIApp(QApplication):
	def __init__(self):
		QApplication.__init__(self,sys.argv)
		self.eventProcessor = EventProcessor(self)
		self.mainWindow = QMainWindow()
		self.setMainWidget(self.mainWindow)
	
	def search(self,files):
		for file in files:
			SyncOperation(self,self._search_sync,file)
	
	def edit_tempo(self,files):
		for file in files:
			SyncOperation(self,self._edit_tempo_sync,file)
	
	def _search_sync(self,file):
		search_window = OSearchWindow(self.mainWindow)
		try:
			title = None
			message = None
			search_window.show()
			search_window.search(file)
		except NotAnalyzedYet,e:
			search_window.reject()
			title = "Cannot search for similar track yet"
			message = tounicode("The track %s has not yet been analyzed, and thus it cannot be compared to other tracks in your collection.  Please right-click the track, select Smart DJ->Analyze selection now, and try again in 30 seconds."%file)
			
		except UnAnalyzable,e:
			search_window.reject()
			title = "Analysis on the selected track failed"
			message = tounicode("The track %s could not be analyzed, probably because its compression format was not understood by songanalysis."%file)
			
		except DcopAmarokFailed,e:
			logger.debug("amaroK has gone unexpectedly",exc_info=True)
			search_window.reject()
			return
			
		if title and message:
			m = QMessageBox(title,message,QMessageBox.Information,QMessageBox.Ok,
			QMessageBox.NoButton,QMessageBox.NoButton,self.mainWindow,None,True)
			m.setTextFormat(Qt.RichText)
			m.show()
	
	def _edit_tempo_sync(self,file):
		tempo_window = OTapTempo(self.mainWindow)
		try:
			title = None
			message = None
			tempo_window.show()
			tempo_window.assignTrack(file)
		
		except DcopAmarokFailed,e:
			logger.debug("amaroK has gone unexpectedly",exc_info=True)
			tempo_window.reject()
			return
	
	def _update_config_window_sync(self):
		existing_child = self.mainWindow.child("Preferences window",None,True)
		if existing_child:
			existing_child.updateStatus()
	
	def update_config_window(self):
		SyncOperation(self,self._update_config_window_sync)
	
	def show_configuration(self):
		SyncOperation(self,self._show_configuration_sync)
			
	def _show_configuration_sync(self):
		existing_child = self.mainWindow.child("Preferences window",None,True)
		if existing_child:
			existing_child.raiseW()
		else:
			OSmartDJPreferences(self.mainWindow).show()


def _run_ui_loop():
	global app
	try:
		logger.debug("Running ui loop in thread: %s",currentThread())
		app = UIApp()
		app.exec_loop()
	except:
		logger.exception("Something went wrong in the UI loop thread")


#------------ public implementation starts ----------------------

custom_menu_items = ["Find tracks similar to selection...","Edit tempo of selected tracks...","Analyze selected tracks now","Set up Auto DJ mode..."]

def initialize(amaroKProxy,configObject,analysisProcessInstance):
	global amarok
	global config
	global analysisProcess
	logger.debug("Initializing UI")
	amarok = amaroKProxy
	config = configObject
	analysisProcess = analysisProcessInstance
	t = Thread(target=_run_ui_loop)
	t.setName("UI thread")
	t.setDaemon(True)
	t.start()
	for a in custom_menu_items:
		amarok.script.addCustomMenuItem("Smart DJ",a)

def end():
	logger.debug("Disabling UI")
	for a in custom_menu_items:
		try: amarok.script.removeCustomMenuItem("Smart DJ",a)
		except DcopAmarokFailed: pass

def search(start_files):
	""" Receive a Python list of file paths (NOT URLs)"""
	while app is None: time.sleep(.3)
	app.search(start_files)

def edit_tempo(file):
	""" Receive a Python string ( a path ) """
	while app is None: time.sleep(.3)
	app.edit_tempo(file)

def show_configuration():
	while app is None: time.sleep(.3)
	app.show_configuration()

